'use strict';

/* Controllers */

var kcRefControllers = angular.module('kcRef.controllers', []);

/* Controller for getting the customer info and allowing name changes */
kcRefControllers.controller('CustomerCtrl', ['$scope', '$rootScope', 'apiProxy', 'kcSession',
	function($scope, $rootScope, apiProxy, kcSession) {

		/* Processing the Angular UI alerts */
		var processAlertMessages = function(issues, alerts, type) {
			angular.forEach(issues, function(issue) {
				alerts.push({type: type, msg: issue.message});
			});
		};
		$scope.closeAlert = function(index) {
			$scope.alerts.splice(index, 1);
		};

		/* Get the customer */
		$scope.getCustomer = function() {
			$scope.alerts = []; /* Clear any alerts */

			/* Make sure the thresholds page is hidden by broadcasting a message to the other controller */
			$rootScope.$broadcast('HIDE_THRESHOLDS');

			/* Make the call to get the customer info through the api proxy service */
			apiProxy.post('getCustomer.json', {customerId: $scope.customerId}).then(function(data) {
				$scope.showCustomerOptions = false;
				kcSession.customerId = '';
				if (data !== undefined && data.hasOwnProperty('errors')) {
					if (data.errors.length) {
						processAlertMessages(data.errors, $scope.alerts, 'danger');
					} else {
						$scope.customer = data.reply.customer;
						$scope.customerName = data.reply.customer.name;
						kcSession.customerId = $scope.customerId;
						/* Send a message to the Thresholds controller to retrieve and show the thresholds */
						$rootScope.$broadcast('SHOW_THRESHOLDS');
						processAlertMessages(data.warnings, $scope.alerts, 'warning');
						$scope.showCustomerOptions = true;
					}
				}
				else {
					processAlertMessages([{message: 'Unable to process response from server'}], $scope.alerts, 'danger');
				}
			});
		};

		/* Change the customer's name */
		$scope.changeName = function() {
			$scope.alerts = [];
			api.post('updateCustomer.json', {customerId: $scope.customerId, customer: $scope.customer}).then(function(data) {
				if (data !== undefined && data.hasOwnProperty('reply')) {
					$scope.customer = data.reply.customer;
					$scope.customerName = data.reply.customer.name;
				} else {
					processAlertMessages([{message: 'Unable to process response from server'}], $scope.alerts, 'danger');
				}
			});
		};
	}]);

/* Controller for displaying and modifying the thresholds */
kcRefControllers.controller('ThresholdsCtrl', ['$scope', 'apiProxy', 'kcSession',
	function($scope, apiProxy, kcSession) {

		/* Processing the Angular UI alerts */
		var processAlertMessages = function(issues, alerts, type) {
			angular.forEach(issues, function(issue) {
				alerts.push({type: type, msg: issue.message});
			});
		};
		$scope.closeAlert = function(index) {
			$scope.alerts.splice(index, 1);
		};

		/* Handle the broadcast messages */
		$scope.$on('SHOW_THRESHOLDS', function() {
			$scope.alerts = [];
			getThresholds();
		});
		$scope.$on('HIDE_THRESHOLDS', function() {
			$scope.showThresholds = false;
			$scope.toggleType = false;
			$scope.numberType = false;
			$scope.listType = false;
		});

		/* Make the call to get the thresholds through the api proxy service */
		var getThresholds = function() {
			apiProxy.post('getThresholds.json', {customerId: kcSession.customerId}).then(function(data) {
				if (data !== undefined && data.hasOwnProperty('reply')) {
					/* Don't show any thresholds initially */
					$scope.categoryFilter = {tags: "__X__"};

					/* Store the thresholds in this rawThresholds object, as this is what will be sent back for an updateThresholds call */
					$scope.rawThresholds = data.reply.thresholds;

					/* Put the thresholds objects in an array so that Angular can handle it for the ng-repeat directive */
					var thresholds = [];
					angular.forEach(data.reply.thresholds, function(threshold) {
						thresholds.push(threshold);
					});
					thresholds.sort(function(a, b) {
						return a.name.localeCompare(b.name);
					});
					$scope.thresholds = thresholds;

					/* Get the categories that the thresholds will be categorized under */
					$scope.categories = getCategories();
					$scope.showThresholds = true;
				} else {
					processAlertMessages([{message: 'Unable to process response from server'}], $scope.alerts, 'danger');
				}
			});
		};

		/* Loop through the tags field in the thresholds to get a list of tags to categorize the thresholds with */
		var getCategories = function() {
			// Form the categories from the tags field for each threshold
			var categoryArray = [];
			angular.forEach($scope.thresholds, function(threshold) {
				if (categoryArray.indexOf(threshold.tags) === -1) {
					categoryArray.push(threshold.tags);
				}
			});
			/* Push an emptry string for the 'All' category */
			categoryArray.push("");
			return categoryArray.sort();
		};

		/* Select the threshold category to filter with */
		$scope.selectCategory = function(category) {
			$scope.alerts = [];
			$scope.categoryFilter = {tags: category};
			$scope.toggleType = false;
			$scope.numberType = false;
			$scope.listType = false;
			$scope.threshold = null;
		};

		/* Helper method to select all or deselect all list threshold options */
		$scope.setAllOptions = function(selected) {
			angular.forEach($scope.listOptions, function(option) {
				option.selected = selected;
			});
		};

		/* Sets up a threshold to be edited when selected from the list of thresholds */
		$scope.selectThreshold = function(threshold) {
			$scope.alerts = [];
			$scope.threshold = threshold;
			$scope.displayName = threshold.name;
			$scope.displayDescription = threshold.description;

			/* Clear out the type of the threshold */
			$scope.toggleType = false;
			$scope.numberType = false;
			$scope.listType = false;

			/* Set up individual threshold types */
			$scope.toggleValue = threshold.enabled; /* All thresholds have enabled property */

			/* Set up for the type of the threshold to be configured  */
			if ($scope.threshold.type === 'toggle') {
				$scope.toggleType = true;
			}
			else if ($scope.threshold.type === 'number') {
				$scope.numberType = true;
				$scope.numberValue = $scope.threshold.value;
			}
			else if ($scope.threshold.type === 'list') {
				$scope.listType = true;

				/* Create an array of options from the json options object */
				/* This is done so that angular can work with the array through directives */
				var options = [];
				angular.forEach($scope.threshold.options, function(name, id) {
					options.push({id: id, name: name});
				});
				options.sort(function(a, b) {
					return a.name.localeCompare(b.name);
				});
				$scope.listOptions = options;

				/* Go through the values set for the list threshold and set the selected  
				 * property for each corresponding option in the list presented to the user */
				angular.forEach($scope.threshold.value, function(id) {
					angular.forEach($scope.listOptions, function(option) {
						if (option.id === id) {
							option.selected = true;
						}
					});
				});
			}
		};

		/* Save the threshold */
		$scope.saveThreshold = function() {
			$scope.alerts = [];

			/* Create a copy of the current threshold and set the values on it to send to the api */
			var thresholdToSave = jQuery.extend(true, {}, $scope.threshold);
			thresholdToSave.enabled = $scope.toggleValue;
			if ($scope.numberType) {
				thresholdToSave.value = $scope.numberValue ? $scope.numberValue : 0;
			}
			else if ($scope.listType) {
				var values = [];
				for (var index = 0; index < $scope.listOptions.length; index++) {
					if ($scope.listOptions[index].selected) {
						values.push($scope.listOptions[index].id);
					}
				}
				thresholdToSave.value = values;
			}

			/* Create the data to send to the api, and only send the threshold being edited */
			var kcData = {customerId: kcSession.customerId, thresholds: {}};
			kcData.thresholds[thresholdToSave.code] = thresholdToSave;

			/* Send the update to the api */
			apiProxy.post('updateThresholds.json', kcData).then(function(data) {
				if (data !== undefined && data.hasOwnProperty('reply')) {
					if (data.errors.length) {
						processAlertMessages(data.errors, $scope.alerts, 'danger');
					}
					else {
						$scope.alerts.push({type: 'success', msg: 'Threshold saved'});
						processAlertMessages(data.warnings, $scope.alerts, 'warning');

						/* Update the current threshold data with the returned data to keep the model and view up-to-date */
						$scope.threshold.enabled = data.reply.thresholds[$scope.threshold.code].enabled;
						if (!$scope.toggleType) {
							$scope.threshold.value = data.reply.thresholds[$scope.threshold.code].value;
						}
					}
				} else {
					processAlertMessages([{message: 'Unable to process response from server'}], $scope.alerts, 'danger');
				}
			});
		};
	}]);
